C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      SUBROUTINE DB_TRAN_MODISD042(FILENAME, TEMP, WVMR, OZMR, THETA,
     &    KBAN, JDET, TAUT)

c-----------------------------------------------------------------------
c!F77
c
c!Description:
c    Terra/Aqua MODIS 42-level fast transmittance routine.
c    LarrabeeStrow/HalWoolf/PaulVanDelst regression model based on
c    LBLRTM line-by-line transmittances.
c
c!Input Parameters:
c    FILENAME Configuration file name
c    TEMP     Temperatures at 42 standard levels (Kelvin)
c    WVMR     Water vapor mixing ratios at 42 standard levels (g/kg)
c    OZMR     Ozone concentrations at 42 standard levels (ppmv) 
c    THETA    Local zenith angle from surface to satellite (degrees)
c    KBAN     MODIS IR band number (20-25, 27-36)
c             (for band 26, return TAUT array is filled with 1.0)
c    JDET     Detector number (0-10; product order)
c             (detector zero is based on the average spectral response)
c
c    Input temperatures, and water-vapor and ozone mixing ratios, must
c    be defined at the pressure levels in array 'pstd'
c    (see block data 'reference_atmosphere' in file 'irtsub_p42.f')
c
c!Output Parameters:
c    TAUT     Total transmittances (no units) at 42 standard levels
c
c!Revision History:
c
c!Team-unique Header:
c
c!End
c-----------------------------------------------------------------------

      implicit none

c ... UW include files
      include 'platform_name.inc'
      include 'cloudtop.inc'
            
c ... Parameters
      integer nd
      parameter (nd = 10)
      integer nk
      parameter (nk = 5)
      integer nl
      parameter (nl = 42)
      integer nm
      parameter (nm = nl-1)
      integer koff
      parameter (koff = 19)
      integer nr
      parameter (nr = 17)
      integer nxc
      parameter (nxc = 4)
      integer ncc
      parameter (ncc = nxc+1)
      integer lencc
      parameter (lencc = ncc*nm)
      integer lenccb
      parameter (lenccb = lencc*4)
      integer nxd
      parameter (nxd = 8)
      integer ncd
      parameter (ncd = nxd+1)
      integer lencd
      parameter (lencd = ncd*nm)
      integer lencdb
      parameter (lencdb = lencd*4)
      integer nxo
      parameter (nxo = 9)
      integer nco
      parameter (nco = nxo+1)
      integer lenco
      parameter (lenco = nco*nm)
      integer lencob
      parameter (lencob = lenco*4)
      integer nxl
      parameter (nxl = 2)
      integer ncl
      parameter (ncl = nxl+1)
      integer lencl
      parameter (lencl = ncl*nm)
      integer lenclb
      parameter (lenclb = lencl*4)
      integer nxs
      parameter (nxs = 11)
      integer ncs
      parameter (ncs = nxs+1)
      integer lencs
      parameter (lencs = ncs*nm)
      integer lencsb
      parameter (lencsb = lencs*4)
      integer ndt
      parameter (ndt = nd+1)
      integer nrps
      parameter (nrps = nr*ndt)
      integer nxw
      parameter (nxw = nxl+nxs)
      integer header_len
      parameter (header_len = 3)

c ... Argument variables
      real temp(*), wvmr(*), ozmr(*), theta, taut(*)
      integer kban, jdet
      character*(*) filename

c...  Common variables
      real ostd(nl), pstd(nl), tstd(nl), wstd(nl)
      common /atmstd/ pstd, tstd, wstd, ostd

      real taud(nl), tauo(nl), tauw(nl)
      common /taudwo/ taud, tauw, tauo

c ... Local variables
      character*3 comp(nk)
      character*255 file
      character*80 keyname
      integer keyindex,status,io_err

      integer i, j, k, l, ikrec, krec, krecx, ksat
      integer create_date, ios, init
      
      logical newang, newatm

      real coefc(ncc,nm,0:nd,nr), coefd(ncd,nm,0:nd,nr)
      real coefl(ncl,nm,0:nd,nr), coefo(nco,nm,0:nd,nr)
      real coefs(ncs,nm,0:nd,nr)
      real datm, do, dt, dw, oamt(nm), olas(nl)
      real oref(nm), pavg(nm), secz(nm), tauc(nl), tavg(nm), tlas(nl)
      real tref(nm), wamt(nm), wlas(nl), wref(nm), xcon(nxc,nm)
      real xdry(nxd,nm), xozo(nxo,nm), xwet(nxw,nm), z, zlas, zsec
      real header(header_len)

c ... Statement functions
      real secant

c ... Save statement is REQUIRED in this subroutine
c ... since coefficients are read on first pass only
      save 

c ... Data statements
      data tlas/nl * 0.0/, wlas/nl * 0.0/, olas/nl * 0.0/, zlas/-999.0/
      data comp/'dry', 'ozo', 'wts', 'wtl', 'wco'/
      data init/1/

c ... External routines
      external hdrgetkeystr,sbflip,big_endian
      integer hdrgetkeystr
      real sbflip
      logical big_endian

c ... Statement function definition
      secant(z) = 1.0 / cos(0.01745329 * z)

c-----------------------------------------------------------------------
c     INITIALIZATION (FIRST PASS ONLY)
c-----------------------------------------------------------------------

c ... Check if initialization is required
      if (init .eq. 1) then

c ...   Check the platform name      
        if     (platform_name(1:5) .eq. 'TERRA' .or.
     &          platform_name(1:5) .eq. 'Terra' .or.
     &          platform_name(1:5) .eq. 'terra') then
          ksat=1
        elseif (platform_name(1:4) .eq. 'AQUA' .or.
     &          platform_name(1:4) .eq. 'Aqua' .or.
     &          platform_name(1:4) .eq. 'aqua') then
          ksat=2
        else
          call message('tran_modisd042.f',
     &      'Platform name not recognized ' ,
     &       0, 2)
        endif

c ...   Get the record number for band 26, detector zero
        ikrec = nrps * (ksat - 1)
        krecx = ikrec + 7

c ...   Set the creation date for supported coefficient files
        create_date = 2002009


c ....  DRY GAS FILE ..........................................
c ...   Open and check the coefficient files
        file = '  '
        keyname = 'DRY_GAS_FILE'
        keyindex = 1
        status = hdrgetkeystr( filename, keyname, keyindex, file )
        if( status.lt.0 ) then
           call message('trans042','Failed reading dry gas file ',
     &                 0, 2)
        else
          open(FILE=file,UNIT=DRY_GAS_UNIT,STATUS=DRY_GAS_STATUS,
     &         FORM=DRY_GAS_FORM,ACCESS=DRY_GAS_ACCESS,
     &         RECL=lencdb,IOSTAT=io_err)
          if (io_err .ne. 0) then
           call message('trans042','Failed opening dry gas file ',
     &                 0, 2)
          endif
        endif
        read(DRY_GAS_UNIT,rec=krecx) header
        if (.not. big_endian()) then
            do i = 1 , header_len
                 header(i) = sbflip(header(i))
            enddo
        endif
c ...   Check the header record for band 26, detector zero
c ...   (word 1: Satellite number)
c ...   (word 2: Number of levels)
c ...   (word 3: Creation date YYYYDDD)
        if (nint(header(1)) .ne. ksat .or.
     &      nint(header(2)) .ne. nl .or.
     &      nint(header(3)) .ne. create_date) then
          call message('tran_modisd042.f',
     &      'Invalid coefficient file ' // comp(1) , 
     &      0, 2)
        endif

c ...   Now read in the dry gas coefficients
        krec = ikrec
        do l = 0, nd
          do k = 1, nr
            krec = krec + 1
            read(DRY_GAS_UNIT,rec=krec,iostat=ios) 
     &          ((coefd(i,j,l,k),i=1,ncd),j=1,nm)
            if (ios .ne. 0) then
                call message('tran_modisd042.f',
     &          'Error reading coefficient file ' // comp(1) ,
     &           0, 2) 
            endif
          enddo
        enddo
        close(DRY_GAS_UNIT)

c ...   Check if a little endian machine.  If so, then flip bytes
        if (.not. big_endian()) then
          do k = 1 , nr
            do l = 0 , nd
              do j = 1 , nm
                 do i = 1 , ncd
                    coefd(i,j,l,k) = sbflip(coefd(i,j,l,k))
                 enddo
              enddo
            enddo
          enddo
        endif
c ...................................................................


c ....  OZONEFILE ...................................................
c ...   Open and check the coefficient files
        keyname = 'OZONE_FILE'
        keyindex = 1
        status = hdrgetkeystr( filename, keyname, keyindex, file )
        if( status.lt.0 ) then
           call message('trans042','Failed reading ozone coeff. file ',
     &                 0, 2)
        else
          open(FILE=file,UNIT=OZONE_UNIT,STATUS=OZONE_STATUS,
     &         FORM=OZONE_FORM,ACCESS=OZONE_ACCESS,
     &         RECL=lencob,IOSTAT=io_err)
          if (io_err .ne. 0) then
           call message('trans042','Failed opening ozone gas file ',
     &                 0, 2)
          endif
        endif
        read(OZONE_UNIT,rec=krecx) header
        if (.not. big_endian()) then
            do i = 1 , header_len
                 header(i) = sbflip(header(i))
            enddo
        endif
c ...   Check the header record for band 26, detector zero
c ...   (word 1: Satellite number)
c ...   (word 2: Number of levels)
c ...   (word 3: Creation date YYYYDDD)
        if (nint(header(1)) .ne. ksat .or.
     &      nint(header(2)) .ne. nl .or.
     &      nint(header(3)) .ne. create_date) then
          call message('tran_modisd042.f',
     &      'Invalid coefficient file ' // comp(2) ,
     &       0, 2)
        endif

c ...   Now read in the ozone coefficients
        krec = ikrec
        do l = 0, nd
          do k = 1, nr
            krec = krec + 1
            read(OZONE_UNIT,rec=krec,iostat=ios) 
     &         ((coefo(i,j,l,k),i=1,nco),j=1,nm)
            if (ios .ne. 0) then
                call message('tran_modisd042.f',
     &          'Error reading coefficient file ' // comp(2) ,
     &           0, 2)                                 
            endif
          enddo
        enddo
        close(OZONE_UNIT)

c ...   Check if a little endian machine.  If so, then flip bytes
        if (.not. big_endian()) then
          do k = 1 , nr
            do l = 0 , nd
              do j = 1 , nm
                 do i = 1 , nco
                    coefo(i,j,l,k) = sbflip(coefo(i,j,l,k))
                 enddo
              enddo
            enddo
          enddo
        endif
c ...................................................................

c ....  WET ATMOSPHERE FILE 1 .......................................
c ...   Open and check the coefficient files
        keyname = 'WET1_FILE'
        keyindex = 1
        status = hdrgetkeystr( filename, keyname, keyindex, file )
        if( status.lt.0 ) then
           call message('trans042','Failed reading wet1 coeff. file ',
     &                 0, 2)
        else
          open(FILE=file,UNIT=WET1_UNIT,STATUS=WET1_STATUS,
     &         FORM=WET1_FORM,ACCESS=WET1_ACCESS,
     &         RECL=lencsb,IOSTAT=io_err)
          if (io_err .ne. 0) then
           call message('trans042','Failed opening wet1 gas file ',
     &                 0, 2)
          endif
        endif
        read(WET1_UNIT,rec=krecx) header
        if (.not. big_endian()) then
            do i = 1 , header_len
                 header(i) = sbflip(header(i))
            enddo
        endif
c ...   Check the header record for band 26, detector zero
c ...   (word 1: Satellite number)
c ...   (word 2: Number of levels)
c ...   (word 3: Creation date YYYYDDD)
        if (nint(header(1)) .ne. ksat .or.
     &      nint(header(2)) .ne. nl .or.
     &      nint(header(3)) .ne. create_date) then
          call message('tran_modisd042.f',
     &      'Invalid coefficient file ' // comp(3) ,
     &       0, 2)
        endif

c ...   Now read in the wet  gas coefficients file 1
        krec = ikrec
        do l = 0, nd
          do k = 1, nr
            krec = krec + 1
            read(WET1_UNIT,rec=krec,iostat=ios) 
     &          ((coefs(i,j,l,k),i=1,ncs),j=1,nm)
            if (ios .ne. 0) then
                call message('tran_modisd042.f',
     &          'Error reading coefficient file ' // comp(3) ,
     &           0, 2)                                 
            endif
          enddo
        enddo
        close(WET1_UNIT)

c ...   Check if a little endian machine.  If so, then flip bytes
        if (.not. big_endian()) then
          do k = 1 , nr
            do l = 0 , nd
              do j = 1 , nm
                 do i = 1 , ncs
                    coefs(i,j,l,k) = sbflip(coefs(i,j,l,k))
                 enddo
              enddo
            enddo
          enddo
        endif
c ...................................................................

c ....  WET ATMOSPHERE FILE 2 .......................................
c ...   Open and check the coefficient files
        keyname = 'WET2_FILE'
        keyindex = 1
        status = hdrgetkeystr( filename, keyname, keyindex, file )
        if( status.lt.0 ) then
           call message('trans042','Failed reading wet2 coeff. file ',
     &                 0, 2)
        else
          open(FILE=file,UNIT=WET2_UNIT,STATUS=WET2_STATUS,
     &         FORM=WET2_FORM,ACCESS=WET2_ACCESS,
     &         RECL=lenclb,IOSTAT=io_err)
          if (io_err .ne. 0) then
           call message('trans042','Failed opening wet2 gas file ',
     &                 0, 2)
          endif
        endif
        read(WET2_UNIT,rec=krecx) header
        if (.not. big_endian()) then
            do i = 1 , header_len
                 header(i) = sbflip(header(i))
            enddo
        endif
c ...   Check the header record for band 26, detector zero
c ...   (word 1: Satellite number)
c ...   (word 2: Number of levels)
c ...   (word 3: Creation date YYYYDDD)
        if (nint(header(1)) .ne. ksat .or.
     &      nint(header(2)) .ne. nl .or.
     &      nint(header(3)) .ne. create_date) then
          call message('tran_modisd042.f',
     &      'Invalid coefficient file ' // comp(4) ,
     &       0, 2)
        endif

c ...   Now read in the wet gas coefficients file 2
        krec = ikrec
        do l = 0, nd
          do k = 1, nr
            krec = krec + 1
            read(WET2_UNIT,rec=krec,iostat=ios) 
     &           ((coefl(i,j,l,k),i=1,ncl),j=1,nm)
            if (ios .ne. 0) then
                call message('tran_modisd042.f',
     &          'Error reading coefficient file ' // comp(4) ,
     &           0, 2)
            endif
          enddo
        enddo
        close(WET2_UNIT)

c ...   Check if a little endian machine.  If so, then flip bytes
        if (.not. big_endian()) then
          do k = 1 , nr
            do l = 0 , nd
              do j = 1 , nm
                 do i = 1 , ncl
                    coefl(i,j,l,k) = sbflip(coefl(i,j,l,k))
                 enddo
              enddo
            enddo
         enddo
        endif
c ...................................................................

c ....  WATER VAPOR FILE ............................................
c ...   Open and check the coefficient files
        keyname = 'WATER_VAPOR_FILE'
        keyindex = 1
        status = hdrgetkeystr( filename, keyname, keyindex, file )
        if( status.lt.0 ) then
           call message('trans042','Failed reading wv coeff. file ',
     &                 0, 2)
        else
         open(FILE=file,UNIT=WATER_VAPOR_UNIT,STATUS=WATER_VAPOR_STATUS,
     &         FORM=WATER_VAPOR_FORM,ACCESS=WATER_VAPOR_ACCESS,
     &         RECL=lenccb,IOSTAT=io_err)
          if (io_err .ne. 0) then
           call message('trans042','Failed opening water vapor file ',
     &                 0, 2)
          endif
        endif
        read(WATER_VAPOR_UNIT,rec=krecx) header
        if (.not. big_endian()) then
            do i = 1 , header_len
                 header(i) = sbflip(header(i))
            enddo
        endif
c ...   Check the header record for band 26, detector zero
c ...   (word 1: Satellite number)
c ...   (word 2: Number of levels)
c ...   (word 3: Creation date YYYYDDD)
        if (nint(header(1)) .ne. ksat .or.
     &      nint(header(2)) .ne. nl .or.
     &      nint(header(3)) .ne. create_date) then
          call message('tran_modisd042.f',
     &      'Invalid coefficient file ' // comp(5) ,
     &       0, 2)
        endif

c ...   Now read in the water vapor coefficients
        krec = ikrec
        do l = 0, nd
          do k = 1, nr
            krec = krec + 1
            read(WATER_VAPOR_UNIT,rec=krec,iostat=ios) 
     &          ((coefc(i,j,l,k),i=1,ncc),j=1,nm)
            if (ios .ne. 0) then
                call message('tran_modisd042.f',
     &          'Error reading coefficient file ' // comp(5) ,
     &           0, 2)
            endif
          enddo
        enddo
        close(WATER_VAPOR_UNIT)

c ...   Check if a little endian machine.  If so, then flip bytes
        if (.not. big_endian()) then
          do k = 1 , nr
            do l = 0 , nd 
              do j = 1 , nm
                 do i = 1 , ncc
                    coefc(i,j,l,k) = sbflip(coefc(i,j,l,k))
                 enddo
              enddo
           enddo
          enddo
        endif
c ...................................................................
        
c ...   Convert standard water vapor and ozone profiles
        i = 1
        call conpir(pstd, tstd, wstd, ostd, nl, i, pavg,
     &    tref, wref, oref)

c ...   Save flag indicating initialization is complete
        init = 0

      endif

c-----------------------------------------------------------------------
c     END OF INITIALIZATION
c-----------------------------------------------------------------------

      dt = 0.0
      dw = 0.0
      do = 0.0
      do j = 1, nl
        dt = dt + abs(temp(j) - tlas(j))
        tlas(j) = temp(j)
        dw = dw + abs(wvmr(j) - wlas(j))
        wlas(j) = wvmr(j)
        do = do + abs(ozmr(j) - olas(j))
        olas(j) = ozmr(j)
        taud(j) = 1.0
        tauw(j) = 1.0
        tauc(j) = 1.0
        tauo(j) = 1.0
        taut(j) = 1.0
      enddo
      datm = dt + dw + do
      
c ... Convert profile
      newatm = (abs(datm) .gt. 1.0e-6)
      if(newatm) then
        i = 1 
        call conpir(pstd, temp, wvmr, ozmr, nl, i, pavg,
     &    tavg, wamt, oamt)
      endif

c ... Set zenith
      newang = (abs(theta - zlas) .gt. 1.0e-6)
      if(newang) then
         zsec = secant(theta)
         do l= 1, nm
            secz(l) = zsec
         enddo
         zlas = theta
      endif

c ... Compute predictors
      if(newang .or. newatm) then
        call calpir(tref, wref, oref, tavg, wamt, oamt, pavg, secz,
     &    nm, nxd, nxw, nxo, nxc, xdry, xwet, xozo, xcon)
      endif

c ... If band 26 is requested, set transmittances to 1.0 and return
      if(kban .eq. 26) then
        do j = 1, nl
          taud(j) = 1.0
          tauo(j) = 1.0
          tauw(j) = 1.0
          taut(j) = 1.0
        enddo
        return
      endif

c ... Compute detector and band indexes
      j = jdet
      k = kban - koff

c ... Compute dry component
      call taudoc(ncd, nxd, nm, coefd(1, 1, j, k), xdry, taud)

c ... Compute ozone component
      call taudoc(nco, nxo, nm, coefo(1, 1, j, k), xozo, tauo)

c ... Compute wet component
      call tauwtr(ncs, ncl, nxs, nxl, nxw, nm, coefs(1, 1, j, k),
     &  coefl(1, 1, j, k), xwet, tauw)
      call taudoc(ncc, nxc, nm, coefc(1, 1, j, k), xcon, tauc)
      do j = 1, nl
        tauw(j) = tauw(j) * tauc(j)
      enddo

c ... Compute total transmittance
      do j = 1, nl
        taut(j) = taud(j) * tauo(j) * tauw(j)
      enddo

      END

c * Subprograms for infrared transmittance
c       at 42-level SPACECRAFT pressure coordinate

c .... version of 10.01.02

contents:

c     block data reference_atmosphere
c     subroutine calpir
c     subroutine conpir
c     subroutine gphite
c     subroutine taudoc
c     subroutine tauwtr

c *********************

c!F77
c!Description:
c!Input Parameters:
c!Output Parameters:
c!Revision History:
c!Team-unique Header:
c!End

c ... 1976 U.S. Standard
c ... parameters ..

      block data reference_atmosphere

      integer nl
      parameter (nl=42)
c ...
c ... arrays in common ..
      real pstd(nl),tstd(nl),wstd(nl),ostd(nl)
c ...
c ... common blocks ..
      common /atmstd/pstd,tstd,wstd,ostd
      save /atmstd/
c ...
c ... data statements ..

      data pstd/ .1,.2,.5,1.,1.5,2.,3.,4.,5.,7.,10.,15.,20.,25.,30.,
     + 50.,60.,70.,85.,100.,115.,135.,150.,200.,250.,300.,350.,400.,
     + 430.,475.,500.,570.,620.,670.,700.,780.,850.,920.,950.,1000.,
     + 1025.,1050./

      data tstd/231.70, 245.22,
     +  263.35, 270.63, 264.07, 257.93, 249.51, 243.65, 239.24, 232.64,
     +  228.07, 225.00, 223.13, 221.72, 220.54, 217.28, 216.70, 216.70,
     +  216.70, 216.70, 216.70, 216.70, 216.70, 216.72, 220.85, 228.58,
     +  235.38, 241.45, 244.81, 249.48, 251.95, 258.32, 262.48, 266.40,
     +  268.61, 274.21, 278.74, 282.97, 284.71, 287.50, 288.84, 290.15/

      data wstd/ 0.003,  0.003,
     +   0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003,
     +   0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003,
     +   0.003,  0.003,  0.003,  0.004,  0.005,  0.014,  0.036,  0.089,
     +   0.212,  0.331,  0.427,  0.588,  0.699,  1.059,  1.368,  1.752,
     +   1.969,  2.741,  3.366,  3.976,  4.255,  4.701,  4.916,  5.125/

      data ostd/.65318,1.04797,
     + 2.13548,3.82386,5.26768,6.11313,7.35964,7.75004,7.82119,7.56126,
     + 6.92006,6.10266,5.55513,5.15298,4.59906,2.86792,2.29259,1.80627,
     + 1.28988,0.93973,0.72277,0.54848,0.46009,0.29116,0.16277,0.09861,
     + 0.06369,0.05193,0.04718,0.04097,0.03966,0.03614,0.03384,0.03342,
     + 0.03319,0.03249,0.03070,0.02878,0.02805,0.02689,0.02633,0.02579/
      end

cccccccccc

      subroutine calpir(t_avg_ref,  amt_wet_ref, amt_ozo_ref,
     +                        t_avg,      amt_wet,     amt_ozo,
     +                        p_avg,      sec_theta,   n_layers,
     +                        n_dry_pred, n_wet_pred,  n_ozo_pred,
     +                        n_con_pred,
     +                        pred_dry,   pred_wet,    pred_ozo,
     +                        pred_con)

c!F77
c!Description:
c!Input Parameters:
c!Output Parameters:
c!Revision History:
c!Team-unique Header:
c!End

c  PURPOSE:

c    Routine to calculate the predictors for the dry (temperature), 
c      wet and ozone components of a fast transmittance model for a
c      scanning satellite based instrument.

c  REFERENCES:

c    AIRS FTC package science notes and software, S. Hannon and L. Strow,
c      Uni. of Maryland, Baltimore County (UMBC)

c  CREATED:

c    19-Sep-1996 HMW

c  ARGUMENTS:

c      Input
c    -----------
c     t_avg_ref  - REAL*4 reference layer average temperature array (K)

c    amt_wet_ref - REAL*4 reference water vapour amount array (k.mol)/cm^2

c    amt_ozo_ref - REAL*4 reference ozone amount array (k.mol)/cm^2

c      t_avg     - REAL*4 layer average temperature array (K)

c     amt_wet    - REAL*4 water vapour amount array (k.mol)/cm^2

c     amt_ozo    - REAL*4 ozone amount array (k.mol)/cm^2

c      p_avg     - REAL*4 layer average pressure array (mb)

c    sec_theta   - REAL*4 secant of the zenith angle array

c     n_layers   - INT*4 Number of atmospheric layers

c    n_dry_pred  - INT*4 number of dry (temperature) predictors

c    n_wet_pred  - INT*4 number of water vapour predictors

c    n_ozo_pred  - INT*4 number of ozone predictors

c    n_con_pred  - INT*4 number of water vapour continuum predictors

c      Output
c    -----------
c     pred_dry   - REAL*4 dry gas (temperature) predictor matrix

c     pred_wet   - REAL*4 water vapour predictor matrix

c     pred_ozo   - REAL*4 ozone predictor matrix

c     pred_con   - REAL*4 water vapour continuum predictor matrix

c  COMMENTS:

c    Levels or Layers?
c    -----------------
c      Profile data is input at a number of *LAYERS*.

c    Layer Numbering pt. A
c    ---------------------
c      Layer 1   => Atmosphere between LEVELs 1 & 2
c      Layer 2   => Atmosphere between LEVELs 2 & 3
c                        .
c                        .
c                        .
c      Layer L-1 => Atmosphere between LEVELs L-1 & L

c    Layer Numbering pt. B
c    ---------------------
c      For the HIS instrument, Layer 1 is at the top of the atmosphere
c        and Layer L-1 is at the surface.    

c    Layer Numbering pt. C
c    ---------------------
c      In this routine the number of *LAYERS* is passed in the argument
c        list, _not_ the number of LEVELS.  This was done to improve
c        the readability of this code, i.e. loop from 1->L(ayers) 
c        rather than from 1->L(evels)-1.

c=======================================================================

c-----------------------------------------------------------------------
c                 Turn off implicit type declaration
c-----------------------------------------------------------------------

       implicit none

c------------------------------------------------------------------------
c                             Arguments
c------------------------------------------------------------------------

c -- Input

      integer*4 n_layers,
     +          n_dry_pred, n_wet_pred, n_ozo_pred, n_con_pred

      real*4    t_avg_ref(*), amt_wet_ref(*), amt_ozo_ref(*),
     +          t_avg(*),     amt_wet(*),     amt_ozo(*),
     +          p_avg(*),     sec_theta(*)

c -- Output

      real*4    pred_dry(n_dry_pred, *),
     +          pred_wet(n_wet_pred, *),
     +          pred_ozo(n_ozo_pred, *),
     +          pred_con(n_con_pred, *)

c------------------------------------------------------------------------
c                           Local variables
c------------------------------------------------------------------------

c -- Parameters

      integer*4 max_layers
      parameter ( max_layers = 41 )

      integer*4 max_dry_pred, max_wet_pred, max_ozo_pred, max_con_pred
      parameter ( max_dry_pred = 8,
     +            max_wet_pred = 13,
     +            max_ozo_pred = 9,
     +            max_con_pred = 4 )

c -- Scalars

      integer*4 l

c -- Arrays

c     ....Pressure
      real*4    p_dp(max_layers),
     +          p_norm(max_layers)

c     ....Temperature
      real*4    delta_t(max_layers),
     +          t_ratio(max_layers),
     +          pw_t_ratio(max_layers)      ! Pressure weighted

c     ....Water vapour
      real*4    wet_ratio(max_layers),
     +          pw_wet(max_layers),         ! Pressure weighted
     +          pw_wet_ref(max_layers),     ! Pressure weighted
     +          pw_wet_ratio(max_layers)    ! Pressure weighted

c     ....Ozone
      real*4    ozo_ratio(max_layers), 
     +          pw_ozo_ratio(max_layers),   ! Pressure weighted
     +          pow_t_ratio(max_layers)     ! Pressure/ozone weighted

        save

c************************************************************************
c                         ** Executable code **
c************************************************************************

c------------------------------------------------------------------------
c                   -- Check that n_layers is o.k. --
c------------------------------------------------------------------------

      if( n_layers .gt. max_layers )then
        write(*,'(/10x,''*** calpir : n_layers > max_layers'')')
        stop
      end if 

c------------------------------------------------------------------------
c         -- Check that numbers of predictors is consistent --
c------------------------------------------------------------------------

c     ---------------------------------
c     # of dry (temperature) predictors
c     ---------------------------------

      if( n_dry_pred .ne. max_dry_pred )then
        write(*,'(/10x,''*** calpir : invalid n_dry_pred'')')
        stop
      end if 

c     ----------------------------
c     # of water vapour predictors
c     ----------------------------

      if( n_wet_pred .ne. max_wet_pred )then
        write(*,'(/10x,''*** calpir : invalid n_wet_pred'')')
        stop
      end if 

c     ---------------------
c     # of ozone predictors
c     ---------------------

      if( n_ozo_pred .ne. max_ozo_pred )then
        write(*,'(/10x,''*** calpir : invalid n_ozo_pred'')')
        stop
      end if 

c     --------------------------------------
c     # of water vapour continuum predictors
c     --------------------------------------

      if( n_con_pred .ne. max_con_pred )then
        write(*,'(/10x,''*** calpir : invalid n_con_pred'')')
        stop
      end if 

c------------------------------------------------------------------------
c         -- Calculate ratios, offsets, etc, for top layer --
c------------------------------------------------------------------------

c     ------------------
c     Pressure variables
c     ------------------

      p_dp(1)   = p_avg(1) * ( p_avg(2) - p_avg(1) )
      p_norm(1) = 0.0

c     ---------------------
c     Temperature variables
c     ---------------------

      delta_t(1)    = t_avg(1) - t_avg_ref(1)
      t_ratio(1)    = t_avg(1) / t_avg_ref(1)
      pw_t_ratio(1) = 0.0

c     ----------------
c     Amount variables
c     ----------------

c     ....Water vapour
 
      wet_ratio(1)    = amt_wet(1) / amt_wet_ref(1)
      pw_wet(1)       = p_dp(1) * amt_wet(1)
      pw_wet_ref(1)   = p_dp(1) * amt_wet_ref(1)
      pw_wet_ratio(1) = wet_ratio(1)

c     ....Ozone

      ozo_ratio(1)    = amt_ozo(1) / amt_ozo_ref(1)
      pw_ozo_ratio(1) = 0.0
      pow_t_ratio(1)  = 0.0

c------------------------------------------------------------------------
c         -- Calculate ratios, offsets, etc, for all layers --
c------------------------------------------------------------------------

      do l = 2, n_layers

c       ------------------
c       Pressure variables
c       ------------------

        p_dp(l) = p_avg(l) * ( p_avg(l) - p_avg(l-1) )
        p_norm(l) = p_norm(l-1) + p_dp(l)

c       ---------------------
c       Temperature variables
c       ---------------------

        delta_t(l)    = t_avg(l) - t_avg_ref(l)
        t_ratio(l)    = t_avg(l) / t_avg_ref(l)
        pw_t_ratio(l) = pw_t_ratio(l-1) + ( p_dp(l) * t_ratio(l-1) )

c       ----------------
c       Amount variables
c       ----------------

c       ..Water vapour

        wet_ratio(l)  = amt_wet(l) / amt_wet_ref(l)
        pw_wet(l)     = pw_wet(l-1) + ( p_dp(l) * amt_wet(l) )
        pw_wet_ref(l) = pw_wet_ref(l-1) + ( p_dp(l) * amt_wet_ref(l) )
        
c       ..Ozone

        ozo_ratio(l)    = amt_ozo(l) / amt_ozo_ref(l)
        pw_ozo_ratio(l) = pw_ozo_ratio(l-1) +
     +                      ( p_dp(l) * ozo_ratio(l-1) )
        pow_t_ratio(l)  = pow_t_ratio(l-1) +
     +                      ( p_dp(l) * ozo_ratio(l-1) * delta_t(l-1) )

      end do

c------------------------------------------------------------------------
c              -- Scale the pressure dependent variables --
c------------------------------------------------------------------------

      do l = 2, n_layers

        pw_t_ratio(l)   = pw_t_ratio(l) / p_norm(l)
        pw_wet_ratio(l) = pw_wet(l) / pw_wet_ref(l)
        pw_ozo_ratio(l) = pw_ozo_ratio(l) / p_norm(l)
        pow_t_ratio(l)  = pow_t_ratio(l) / p_norm(l)
 
      end do

c------------------------------------------------------------------------
c                     -- Load up predictor arrays --
c------------------------------------------------------------------------

      do l = 1, n_layers

c       ----------------------
c       Temperature predictors
c       ----------------------

        pred_dry(1,l) = sec_theta(l)
        pred_dry(2,l) = sec_theta(l) * sec_theta(l)
        pred_dry(3,l) = sec_theta(l) * t_ratio(l)
        pred_dry(4,l) = pred_dry(3,l) * t_ratio(l)
        pred_dry(5,l) = t_ratio(l)
        pred_dry(6,l) = t_ratio(l) * t_ratio(l)
        pred_dry(7,l) = sec_theta(l) * pw_t_ratio(l)
        pred_dry(8,l) = pred_dry(7,l) / t_ratio(l) 

c       -----------------------
c       Water vapour predictors
c       -----------------------

        pred_wet(1,l)  = sec_theta(l) * wet_ratio(l)
        pred_wet(2,l)  = sqrt( pred_wet(1,l) )
        pred_wet(3,l)  = pred_wet(1,l) * delta_t(l)
        pred_wet(4,l)  = pred_wet(1,l) * pred_wet(1,l)
        pred_wet(5,l)  = abs( delta_t(l) ) * delta_t(l) * pred_wet(1,l)
        pred_wet(6,l)  = pred_wet(1,l) * pred_wet(4,l)
        pred_wet(7,l)  = sec_theta(l) * pw_wet_ratio(l)
        pred_wet(8,l)  = pred_wet(2,l) * delta_t(l)
        pred_wet(9,l)  = sqrt( pred_wet(2,l) )
        pred_wet(10,l) = pred_wet(7,l) * pred_wet(7,l)
        pred_wet(11,l) = sqrt( pred_wet(7,l) )
        pred_wet(12,l) = pred_wet(1,l)
        pred_wet(13,l) = pred_wet(2,l)

c       ----------------
c       Ozone predictors
c       ----------------

        pred_ozo(1,l) = sec_theta(l) * ozo_ratio(l)
        pred_ozo(2,l) = sqrt( pred_ozo(1,l) )
        pred_ozo(3,l) = pred_ozo(1,l) * delta_t(l)
        pred_ozo(4,l) = pred_ozo(1,l) * pred_ozo(1,l)
        pred_ozo(5,l) = pred_ozo(2,l) * delta_t(l)
        pred_ozo(6,l) = sec_theta(l) * pw_ozo_ratio(l)
        pred_ozo(7,l) = sqrt( pred_ozo(6,l) ) * pred_ozo(1,l)
        pred_ozo(8,l) = pred_ozo(1,l) * pred_wet(1,l)
        pred_ozo(9,l) = sec_theta(l) * pow_t_ratio(l) * pred_ozo(1,l)

c       ---------------------------------
c       Water vapour continuum predictors
c       ---------------------------------

        pred_con(1,l) = sec_theta(l) * wet_ratio(l) /
     *              ( t_ratio(l) * t_ratio(l) )  
        pred_con(2,l) = pred_con(1,l) * pred_con(1,l) / sec_theta(l)
        pred_con(3,l) = sec_theta(l) * wet_ratio(l) / t_ratio(l) 
        pred_con(4,l) = pred_con(3,l) * wet_ratio(l)

      end do
         
      end

ccccccccc

      subroutine conpir( p, t, w, o, n_levels, i_dir,
     +                         p_avg, t_avg, w_amt, o_amt)

c!F77
c!Description:
c!Input Parameters:
c!Output Parameters:
c!Revision History:
c!Team-unique Header:
c!End

c  PURPOSE:

c    Function to convert atmospheric water vapour (g/kg) and ozone (ppmv)
c      profiles specified at n_levels layer BOUNDARIES to n_levels-1
c      integrated layer amounts of units (k.moles)/cm^2.  The average
c      LAYER pressure and temperature are also returned.

c  REFERENCES:

c    AIRS LAYERS package science notes, S. Hannon and L. Strow, Uni. of
c      Maryland, Baltimore County (UMBC)

c  CREATED:

c    19-Sep-1996 HMW

c  ARGUMENTS:

c     Input
c    --------
c       p     - REAL*4 pressure array (mb)

c       t     - REAL*4 temperature profile array (K)

c       w     - REAL*4 water vapour profile array (g/kg)

c       o     - REAL*4 ozone profile array (ppmv)

c    n_levels - INT*4 number of elements used in passed arrays

c     i_dir   - INT*4 direction of increasing layer number

c                 i_dir = +1, Level(1) == p(top)         } satellite/AC
c                             Level(n_levels) == p(sfc)  }    case

c                 i_dir = -1, Level(1) == p(sfc)         } ground-based
c                             Level(n_levels) == p(top)  }    case

c     Output
c    --------
c     p_avg   - REAL*4 average LAYER pressure array (mb)

c     t_avg   - REAL*4 average LAYER temperature (K)

c     w_amt   - REAL*4 integrated LAYER water vapour amount array (k.moles)/cm^2

c     o_amt   - REAL*4 integrated LAYER ozone amount array (k.moles)/cm^2

c  ROUTINES:

c    Subroutines:
c    ------------
c      gphite      - calculates geopotential height given profile data.

c    Functions:
c    ----------
c      NONE

c  COMMENTS:

c    Levels or Layers?
c    -----------------
c      Profile data is input at a number of *LEVELS*.  Number densitites
c        are calculated for *LAYERS* that are bounded by these levels.
c        So, for L levels there are L-1 layers.

c    Layer Numbering
c    ---------------
c      Layer 1   => Atmosphere between LEVELs 1 & 2
c      Layer 2   => Atmosphere between LEVELs 2 & 3
c                        .
c                        .
c                        .
c      Layer L-1 => Atmosphere between LEVELs L-1 & L

c=======================================================================

c-----------------------------------------------------------------------
c              -- Prevent implicit typing of variables --
c-----------------------------------------------------------------------

      implicit none

c-----------------------------------------------------------------------
c                           -- Arguments --
c-----------------------------------------------------------------------

c -- Arrays

      real*4    p(*), t(*), w(*), o(*), 
     +          p_avg(*), t_avg(*), w_amt(*), o_amt(*)

c -- Scalars

      integer*4 n_levels, i_dir

c-----------------------------------------------------------------------
c                         -- Local variables --
c-----------------------------------------------------------------------

c -- Parameters

      integer*4 max_levels
      parameter ( max_levels = 42 )         ! Maximum number of levels

      real*4    r_equator, r_polar, r_avg
      parameter ( r_equator = 6.378388e+06, ! Earth radius at equator
     +            r_polar   = 6.356911e+06, ! Earth radius at pole
     +            r_avg     = 0.5*(r_equator+r_polar) )

      real*4    g_sfc
      parameter ( g_sfc = 9.80665 )         ! Gravity at surface

      real*4    rho_ref
      parameter ( rho_ref = 1.2027e-12 )    ! Reference air "density"

      real*4    mw_dryair, mw_h2o, mw_o3
      parameter ( mw_dryair = 28.97,        ! Molec. wgt. of dry air (g/mol)
     +            mw_h2o    = 18.0152,      ! Molec. wgt. of water
     +            mw_o3     = 47.9982 )     ! Molec. wgt. of ozone

      real*4    R_gas, R_air
      parameter ( R_gas = 8.3143,           ! Ideal gas constant (J/mole/K)
     +            R_air = 0.9975*R_gas )    ! Gas constant for air (worst case) 

c -- Scalars

      integer*4 l, l_start, l_end, l_indx

      real*4    rho1, rho2, p1, p2, w1, w2, o1, o2, z1, z2,
     +          c_avg, g_avg, z_avg, w_avg, o_avg,
     +          dz, dp, r_hgt, wg, og, A, B

c -- Arrays

      real*4    z(max_levels),              ! Pressure heights (m)
     +          g(max_levels),              ! Acc. due to gravity (m/s/s)
     +          mw_air(max_levels),         ! Molec. wgt. of air (g/mol)
     +          rho_air(max_levels),        ! air mass density (kg.mol)/m^3
     +          c(max_levels),              ! (kg.mol.K)/(N.m)
     +          w_ppmv(max_levels)          ! h2o LEVEL amount (ppmv)

        save

c***********************************************************************
c                         ** Executable code **
c***********************************************************************

c-----------------------------------------------------------------------
c           -- Calculate initial values of pressure heights --
c-----------------------------------------------------------------------

      call gphite( p, t, w, 0.0, n_levels, i_dir, z)

c-----------------------------------------------------------------------
c      -- Set loop bounds for direction sensitive calculations --
c      -- so loop iterates from surface to the top             --
c-----------------------------------------------------------------------

      if( i_dir .gt. 0 )then

c       --------------------
c       Data stored top down
c       --------------------

        l_start = n_levels
        l_end   = 1

      else

c       ---------------------
c       Data stored bottom up
c       ---------------------

        l_start = 1
        l_end   = n_levels

      end if

c-----------------------------------------------------------------------
c          -- Air molecular mass and density, and gravity --
c          -- as a function of LEVEL                      --
c-----------------------------------------------------------------------

c     -----------------------
c     Loop from bottom to top
c     -----------------------

      do l = l_start, l_end, -1*i_dir

c       ---------------------------------
c       Convert water vapour g/kg -> ppmv
c       ---------------------------------

        w_ppmv(l) = 1.0e+03 * w(l) * mw_dryair / mw_h2o

c       -----------------------------------------
c       Calculate molecular weight of air (g/mol)
c       ----------------------------------------

        mw_air(l) = ( ( 1.0 - (w_ppmv(l)/1.0e+6) ) * mw_dryair ) +
     +              ( ( w_ppmv(l)/1.0e+06 ) * mw_h2o )

c       ----------------
c       Air mass density
c       ----------------

        c(l) = 0.001 * mw_air(l) / R_air    ! 0.001 factor for g -> kg
        rho_air(l) = c(l) * p(l) / t(l)

c       -------
c       Gravity
c       -------

        r_hgt = r_avg + z(l)                !  m
        g(l) = g_sfc -                      !  m/s^2
     +         g_sfc*( 1.0 - ( (r_avg*r_avg)/(r_hgt*r_hgt) ) )

      end do
 
c-----------------------------------------------------------------------
c                        -- LAYER quantities --
c-----------------------------------------------------------------------

c     -----------------------
c     Loop from bottom to top
c     -----------------------

      do l = l_start, l_end+i_dir, -1*i_dir

c       -------------------------------------------------------
c       Determine output array index.  This is done so that the
c       output data is always ordered from 1 -> L-1 regardless
c       of the orientation of the input data.  This is true by
c       default only for the bottom-up case.  For the top down
c       case no correction would give output layers from 2 -> L
c       -------------------------------------------------------

        if( i_dir .gt. 0 )then

          l_indx = l - 1

        else

          l_indx = l

        end if

c       ---------------------------------------
c       Assign current layer boundary densities
c       ---------------------------------------
 
        rho1 = rho_air(l)
        rho2 = rho_air(l-i_dir)
 
c       ---------
c       Average c
c       ---------

        c_avg = ( (rho1*c(l)) + (rho2*c(l-i_dir)) ) / ( rho1 + rho2 )

c       ---------
c       Average t
c       ---------

        t_avg(l_indx) = 
     +          ( (rho1*t(l)) + (rho2*t(l-i_dir)) ) / ( rho1 + rho2 )

c       ---------
c       Average p
c       ---------

        p1 = p(l)
        p2 = p(l-i_dir)

        z1 = z(l)
        z2 = z(l-i_dir)

        dp = p2 - p1

        A = log(p2/p1) / (z2-z1)
        B = p1 / exp(A*z1)

        p_avg(l_indx) = dp / log(p2/p1)

c       ------------------------------------------------
c       LAYER thickness (rather long-winded as it is not
c       assumed the layers are thin) in m. Includes
c       correction for altitude/gravity.
c       ------------------------------------------------

c       ...Initial values
        g_avg = g(l)
        dz = -1.0 * dp * t_avg(l_indx) / ( g_avg*c_avg*p_avg(l_indx) )

c       ...Calculate z_avg
        z_avg = z(l) + ( 0.5*dz )

c       ...Calculate new g_avg
        r_hgt = r_avg + z_avg 
        g_avg = g_sfc - g_sfc*( 1.0 - ( (r_avg*r_avg)/(r_hgt*r_hgt) ) )

c       ...Calculate new dz
        dz = -1.0 * dp * t_avg(l_indx) / ( g_avg*c_avg*p_avg(l_indx) )

c       ----------------------------------------
c       Calculate LAYER amounts for water vapour
c       ----------------------------------------

        w1 = w_ppmv(l)
        w2 = w_ppmv(l-i_dir)

        w_avg =  ( (rho1*w1) + (rho2*w2) ) / ( rho1+rho2 )

        w_amt(l_indx) =
     +       rho_ref * w_avg * dz * p_avg(l_indx) / t_avg(l_indx)

c       ---------------------------------
c       Calculate LAYER amounts for ozone
c       ---------------------------------

        o1 = o(l)
        o2 = o(l-i_dir)

        o_avg =  ( (rho1*o1) + (rho2*o2) ) / ( rho1+rho2 )

        o_amt(l_indx) = 
     +       rho_ref * o_avg * dz * p_avg(l_indx) / t_avg(l_indx)

      end do

      end

ccccccccc

      subroutine gphite( p, t, w, z_sfc, n_levels, i_dir, z)

c!F77
c!Description:
c!Input Parameters:
c!Output Parameters:
c!Revision History:
c!Team-unique Header:
c!End

c .... version of 10.01.02

c PURPOSE:

c  Routine to compute geopotential height given the atmospheric state.
c    Includes virtual temperature adjustment.

c CREATED:

c  19-Sep-1996 Received from Hal Woolf, recoded by Paul van Delst
c  18-May-2000 Logic error related to z_sfc corrected by Hal Woolf

c  ARGUMENTS:

c     Input
c    --------
c       p     - REAL*4 pressure array (mb)

c       t     - REAL*4 temperature profile array (K)

c       w     - REAL*4 water vapour profile array (g/kg)

c     z_sfc   - REAL*4 surface height (m).  0.0 if not known.

c    n_levels - INT*4 number of elements used in passed arrays

c     i_dir   - INT*4 direction of increasing layer number

c                 i_dir = +1, Level(1) == p(top)         } satellite/AC
c                             Level(n_levels) == p(sfc)  }    case

c                 i_dir = -1, Level(1) == p(sfc)         } ground-based
c                             Level(n_levels) == p(top)  }    case

c     Output
c    --------
c       z     - REAL*4 pressure level height array (m)

c COMMENTS:

c   Dimension of height array may not not be the same as that of the
c     input profile data.

c=======================================================================

c-----------------------------------------------------------------------
c              -- Prevent implicit typing of variables --
c-----------------------------------------------------------------------

      implicit none

c-----------------------------------------------------------------------
c                           -- Arguments --
c-----------------------------------------------------------------------

c -- Arrays

      real*4    p(*), t(*), w(*), 
     +          z(*)

c -- Scalars

      integer*4 n_levels, i_dir

      real*4    z_sfc

c-----------------------------------------------------------------------
c                         -- Local variables --
c-----------------------------------------------------------------------

c -- Parameters

      real*4    rog, fac
      parameter ( rog = 29.2898, 
     +            fac = 0.5 * rog )

c -- Scalars

      integer*4 i_start, i_end, l

      real*4    v_lower, v_upper, algp_lower, algp_upper, hgt

        save

c***********************************************************************
c                         ** Executable code **
c***********************************************************************

c-----------------------------------------------------------------------
c  -- Calculate virtual temperature adjustment and exponential       --
c  -- pressure height for level above surface.  Also set integration --
c  -- loop bounds                                                    --
c-----------------------------------------------------------------------

      if( i_dir .gt. 0 )then

c       --------------------
c       Data stored top down
c       --------------------

        v_lower = t(n_levels) * ( 1.0 + ( 0.00061 * w(n_levels) ) )

        algp_lower = alog( p(n_levels) )

        i_start = n_levels-1
        i_end   = 1

      else

c       ---------------------
c       Data stored bottom up
c       ---------------------

        v_lower = t(1) * ( 1.0 + ( 0.00061 * w(1) ) )

        algp_lower = alog( p(1) )

        i_start = 2
        i_end   = n_levels

      end if

c-----------------------------------------------------------------------
c                     -- Assign surface height --
c-----------------------------------------------------------------------

      hgt = z_sfc

c .. Following added 18 May 2000 ... previously, z(n_levels) for downward
c       (usual) case was not defined!

      if(i_dir.gt.0) then
         z(n_levels) = z_sfc
      else
         z(1) = z_sfc
      endif

c .. End of addition

c-----------------------------------------------------------------------
c             -- Loop over layers always from sfc -> top --
c-----------------------------------------------------------------------

      do l = i_start, i_end, -1*i_dir

c       ----------------------------------------------------
c       Apply virtual temperature adjustment for upper level
c       ----------------------------------------------------

        v_upper = t(l)
        if( p(l) .ge. 300.0 )
     +    v_upper = v_upper * ( 1.0 + ( 0.00061 * w(l) ) )

c       ----------------------------------------------------- 
c       Calculate exponential pressure height for upper layer
c       ----------------------------------------------------- 

        algp_upper = alog( p(l) )

c       ----------------
c       Calculate height
c       ----------------

        hgt = hgt + ( fac*(v_upper+v_lower)*(algp_lower-algp_upper) )

c       -------------------------------
c       Overwrite values for next layer
c       -------------------------------

        v_lower = v_upper
        algp_lower = algp_upper

c       ---------------------------------------------
c       Store heights in same direction as other data
c       ---------------------------------------------

        z(l) = hgt

      end do

      end

ccccccccc

        subroutine taudoc(nc,nx,ny,cc,xx,tau)

c!F77
c!Description:
c!Input Parameters:
c!Output Parameters:
c!Revision History:
c!Team-unique Header:
c!End

c * Strow-Woolf model ... for dry, ozo(ne), and wco (water-vapor continuum)
c .... version of 10.01.02

      implicit none
      
C     Argument variables
      integer nc, nx, ny
      real cc(nc,ny), tau(*), xx(nx,ny)

C     Local variables
      integer i, j
      real taul, tauy, trap, yy

      save

      data trap/-999.99/

      taul=1.
      tau(1)=taul

      do j=1,ny
      
c***        if(taul.eq.0.) go to 100
        if (abs(taul) .lt. 1.0e-6) go to 100

        yy=cc(nc,j)

c***        if(yy.eq.trap) then
        if (abs(yy - trap) .lt. 1.0e-6) then
           taul=0.
           go to 100
        endif

        do i=1,nx
           yy=yy+cc(i,j)*xx(i,j)
        enddo

        tauy=taul*exp(-yy)
        taul=min(tauy,taul)
        tau(j+1)=taul

100     continue

      end do
      
      end

ccccccccc

        subroutine tauwtr(ncs,ncl,nxs,nxl,nxw,ny,ccs,ccl,xx,tau)

c!F77
c!Description:
c!Input Parameters:
c!Output Parameters:
c!Revision History:
c!Team-unique Header:
c!End

c * Strow-Woolf model ... for 'wet' (water-vapor other than continuum)
c .... version of 10.01.02

      implicit none
      
c     Argument variables
      integer ncl, ncs, nxl, nxs, nxw, ny
      real ccl(ncl,ny), ccs(ncs,ny), tau(*), xx(nxw,ny)

c     Local variables
      integer i, j
      real odsum, taul, tauy, trap, yy

      save

      data trap/-999.99/

      odsum=0.
      taul=1.
      tau(1)=taul
      
      do j=1,ny

c***        if(taul.eq.0.) go to 100
        if (abs(taul) .lt. 0.0) go to 100

        if(odsum.lt.5.) then
        
           yy=ccs(ncs,j)
           
c***           if(yy.eq.trap) then
           if (abs(yy - trap) .lt. 1.0e-6) then
              taul=0.
              go to 100
           endif
           
           do i=1,nxs
              yy=yy+ccs(i,j)*xx(i,j)
           enddo
           
           odsum=odsum+abs(yy)
           
        else
        
           yy=ccl(ncl,j)
           
c***           if(yy.eq.trap) then
           if (abs(yy - trap) .lt. 1.0e-6) then
              taul=0.
              go to 100
           endif
           
           do i=1,nxl
              yy=yy+ccl(i,j)*xx(i+11,j)
           enddo
           
           odsum=odsum+abs(yy)
           
        endif

        tauy=taul*exp(-yy)
        taul=min(tauy,taul)
        tau(j+1)=taul
 
100     continue

      end do
      
      end
